package com.quiomputing.cronopartes.actividades;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.NavigationView;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.CursorAdapter;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

import com.quiomputing.cronopartes.R;
import com.quiomputing.cronopartes.adapter.AdaptadorOperacionesContextuales;
import com.quiomputing.cronopartes.bd.TareaBD;
import com.quiomputing.cronopartes.constantes.Constantes;
import com.quiomputing.cronopartes.data.Actividad;
import com.quiomputing.cronopartes.data.Proyecto;
import com.quiomputing.cronopartes.data.Tarea;
import com.quiomputing.cronopartes.services.ServicioCronometro;

public class CronoTask extends AppCompatActivity implements NavigationView.OnNavigationItemSelectedListener,
                                                            AdapterView.OnItemClickListener,
                                                            AdapterView.OnItemLongClickListener,
                                                            Constantes {
    Proyecto.Estado estado = Proyecto.Estado.ABIERTO;
    AdaptadorCursorProyectos adaptador;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_crono_task);
        createInstances();
        installListeners();
    }

    protected void createInstances() {
        adaptador = new AdaptadorCursorProyectos(this, new TareaBD(this).getProyectosAbiertos());
    }

    protected void installListeners() {
        ListView listView = (ListView) findViewById(R.id.listViewProyectos);
        listView.setAdapter(adaptador);
        listView.setOnItemClickListener(this);
        listView.setOnItemLongClickListener(this);

        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.add_proyecto);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent i = new Intent(getBaseContext(), EditorProyectos.class);
                startActivityForResult(i, EDITOR_PROYECTOS);
            }
        });

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.setDrawerListener(toggle);
        toggle.syncState();

        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);
    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

//    @Override
//    public boolean onCreateOptionsMenu(Menu menu) {
//        // Inflate the menu; this adds items to the action bar if it is present.
//        getMenuInflater().inflate(R.menu.crono_partes_trabajo, menu);
//        return true;
//    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        // Handle navigation view item clicks here.
        int id = item.getItemId();

        if (id == R.id.nav_opened) {
            estado = Proyecto.Estado.ABIERTO;
            actualizaLista(estado);
        } else if (id == R.id.nav_closed) {
            estado = Proyecto.Estado.CERRADO;
            actualizaLista(estado);
        } else if (id == R.id.nav_tools) {

        } else if (id == R.id.nav_listado_diario) {
	        Intent i = new Intent(this, InformeDiario.class);
	        startActivity(i);
        }
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        drawer.closeDrawer(GravityCompat.START);
        return true;
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        Intent i = new Intent(getBaseContext(), ListadoActividades.class);
        i.putExtra("id_proyecto", id);
        startActivityForResult(i, LISTADO_ACTIVIDADES);
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> parent, View view, int position, final long id) {
        AlertDialog opciones = new AlertDialog.Builder(this)
            .setAdapter(new AdaptadorOperacionesContextuales(this,
                                                             android.R.layout.select_dialog_item,
                                                             android.R.id.text1),
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog,
                                                int selected) {
                                if (selected == 0) {
                                    Intent i = new Intent(getBaseContext(), EditorProyectos.class);
                                    i.putExtra("id_proyecto", id);
                                    startActivityForResult(i, EDITOR_PROYECTOS);
                                } else if (selected == 1) {
                                    TareaBD bd = new TareaBD(getBaseContext());
                                    bd.cerrarProyecto(id);
                                    actualizaLista(estado);
                                } else if (selected == 2) {
                                    TareaBD bd = new TareaBD(getBaseContext());
                                    bd.borrarProyecto(id);
	                                bd.borrarActividadesDeProyecto(id);
	                                bd.borrarTareasDeProyecto(id);
                                    actualizaLista(estado);
                                }
                            }
                        }).create();
        opciones.show();
        return true;
    }

    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (EDITOR_PROYECTOS == requestCode)
            actualizaLista(estado);
    }

    public void actualizaLista(Proyecto.Estado estado) {
        ListView listView = (ListView) findViewById(R.id.listViewProyectos);
        AdaptadorCursorProyectos adaptador = (AdaptadorCursorProyectos) listView.getAdapter();
        adaptador.changeCursor(new TareaBD(this).getProyectosEnEstado(estado));
    }

    protected class AdaptadorCursorProyectos extends CursorAdapter {
        private LayoutInflater inflador;
        TextView tituloView, subtituloView, tareaView;

        public AdaptadorCursorProyectos(Context contexto, Cursor c) {
            super(contexto, c, false);
        }
        @Override
        public View newView(Context contexto, Cursor c, ViewGroup padre) {
            inflador = (LayoutInflater) contexto.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            return inflador.inflate(R.layout.elemento_proyecto, padre,false);
        }

        @Override
        public void bindView(View vista, Context contexto, Cursor c) {
            Button button = (Button) vista.findViewById(R.id.iniciar_tarea);
            button.setOnClickListener(new IniciarTareaButtonHandler(contexto, c));
            tituloView = (TextView) vista.findViewById(R.id.titulo);
            subtituloView = (TextView) vista.findViewById(R.id.subtitulo);
            tareaView = (TextView) vista.findViewById(R.id.tarea);
            tituloView.setText(c.getString(c.getColumnIndex("nombre")));
            subtituloView.setText(c.getString(c.getColumnIndex("descripcion")));
            TareaBD tareaBD = new TareaBD(contexto);
            Tarea tarea = tareaBD.getUltimaTareaDeProyecto(c.getLong(c.getColumnIndex("_id")));
            if (tarea != null) {
                TareaBD bd = new TareaBD(contexto);
                Actividad actividad = bd.getActividad(tarea.getIdActividad());
                tareaView.setText(actividad.getDescripcion());
            }
        }
    }

    protected class IniciarTareaButtonHandler implements View.OnClickListener {

        protected long idProyecto;
        protected Context context;

        public IniciarTareaButtonHandler(Context context, Cursor c) {
            idProyecto = c.getLong(c.getColumnIndex("_id"));
            this.context = context.getApplicationContext();
        }

        @Override
        public void onClick(View v) {
            TareaBD tareaBD = new TareaBD(context);
            Tarea tarea = tareaBD.getUltimaTareaDeProyecto(idProyecto);
            if (tarea != null) {
                stopService(new Intent(context, ServicioCronometro.class));
                Intent i = new Intent(context, ServicioCronometro.class);
                i.putExtra("id_proyecto", idProyecto);
                i.putExtra("id_actividad", tarea.getIdActividad());
                startService(i);
            }
        }
    }

}
