package com.quiomputing.cronopartes.actividades;

import android.app.DatePickerDialog;
import android.content.Context;
import android.database.Cursor;
import android.os.Bundle;
import android.support.v4.widget.CursorAdapter;
import android.support.v7.app.AppCompatActivity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.DatePicker;
import android.widget.ListView;
import android.widget.TextView;

import com.quiomputing.cronopartes.R;
import com.quiomputing.cronopartes.bd.TareaBD;
import com.quiomputing.cronopartes.constantes.Constantes;

import java.util.Calendar;
import java.util.Date;

/**
 * Created by joaqu on 17/11/2015.
 */
public class InformeDiario extends AppCompatActivity implements DatePickerDialog.OnDateSetListener, Constantes {

	private AdaptadorInformeDiario adaptador;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.listado_diario);
		Date date = new Date(System.currentTimeMillis());
		setTitle(String.format("%s %s", getResources().getString(R.string.report), sdf_solo_fecha.format(date)));
		createInstances();
		installListeners();
	}

	protected void createInstances() {
		adaptador = new AdaptadorInformeDiario(this, new TareaBD(this).getTareasAgrupadasPorDia(new Date(System.currentTimeMillis())));
	}

	protected void installListeners() {
		ListView listView = (ListView) findViewById(R.id.listViewInformeDiario);
		listView.setAdapter(adaptador);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.menu_informe_diario, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		int id = item.getItemId();
		if (id == R.id.seleccionar_fecha) {
			Calendar calendar = Calendar.getInstance();
			DatePickerDialog dialog = new DatePickerDialog(this, this, calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH));
			dialog.show();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.YEAR, year);
		cal.set(Calendar.MONTH, monthOfYear);
		cal.set(Calendar.DAY_OF_MONTH, dayOfMonth);
		setTitle(String.format("%s %s", getResources().getString(R.string.report), sdf_solo_fecha.format(cal.getTime())));
		actualizaLista(cal.getTime());
	}

	public void actualizaLista(Date fecha) {
		ListView listView = (ListView) findViewById(R.id.listViewInformeDiario);
		AdaptadorInformeDiario adaptador = (AdaptadorInformeDiario) listView.getAdapter();
		adaptador.changeCursor(new TareaBD(this).getTareasAgrupadasPorDia(fecha));
	}


	protected class AdaptadorInformeDiario extends CursorAdapter {

		private LayoutInflater inflador;
		TextView tituloView, subtituloView, tareaView, tiempoView;

		public AdaptadorInformeDiario(Context contexto, Cursor c) {
			super(contexto, c, false);
		}
		@Override
		public View newView(Context contexto, Cursor c, ViewGroup padre) {
			inflador = (LayoutInflater) contexto.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			return inflador.inflate(R.layout.elemento_informe_diario, padre,false);
		}

		@Override
		public void bindView(View vista, Context contexto, Cursor c) {
			tituloView = (TextView) vista.findViewById(R.id.inf_diario_titulo);
			subtituloView = (TextView) vista.findViewById(R.id.inf_diario_subtitulo);
			tareaView = (TextView) vista.findViewById(R.id.inf_diario_tarea);
			tiempoView = (TextView) vista.findViewById(R.id.inf_diario_tiempo);
			tituloView.setText(c.getString(c.getColumnIndex("nombre")));
			subtituloView.setText(c.getString(c.getColumnIndex("desc_actividad")));
			tareaView.setText(c.getString(c.getColumnIndex("desc_tarea")));
			int tiempo = Integer.valueOf(c.getString(c.getColumnIndex("tiempo")));
			int horas = tiempo / 3600;
			int minutos = (tiempo - (tiempo /3600)) / 60;
			tiempoView.setText(String.format("%02d:%02d", horas, minutos));
		}
	}
}
