package com.quiomputing.cronopartes.actividades;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.widget.CursorAdapter;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

import com.quiomputing.cronopartes.R;
import com.quiomputing.cronopartes.adapter.AdaptadorOperacionesContextuales;
import com.quiomputing.cronopartes.bd.TareaBD;
import com.quiomputing.cronopartes.constantes.Constantes;
import com.quiomputing.cronopartes.data.Proyecto;
import com.quiomputing.cronopartes.services.ServicioCronometro;

/**
 * Created by joaqu on 14/11/2015.
 */
public class ListadoActividades extends AppCompatActivity implements AdapterView.OnItemClickListener,
                                                                     AdapterView.OnItemLongClickListener,
                                                                     Constantes {

	protected AdaptadorCursorActividades adaptador;
	protected Proyecto proyecto;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_actividades_proyecto);
		createInstances();
		installListeners();
		setTitle(String.format("%s %s", getTitle(), proyecto.getNombre()));
	}

	protected void createInstances() {
		Bundle bundle = getIntent().getExtras();
		TareaBD proyectoBD = new TareaBD(this);
		proyecto = proyectoBD.getProyecto(bundle.getLong("id_proyecto"));
		adaptador = new AdaptadorCursorActividades(this, new TareaBD(this).getActividadesAbiertas(proyecto));
	}

	protected void installListeners() {
		ListView listView = (ListView) findViewById(R.id.listViewActividades);
		listView.setAdapter(adaptador);
		listView.setOnItemClickListener(this);
		listView.setOnItemLongClickListener(this);

		FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.add_actividad);
		fab.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				Intent i = new Intent(getBaseContext(), EditorActividades.class);
				i.putExtra("id_proyecto", proyecto.getId());
				i.putExtra("proyecto", proyecto.getNombre());
				startActivityForResult(i, EDITOR_ACTIVIDADES);
			}
		});
	}

	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (EDITOR_ACTIVIDADES == requestCode)
			actualizaLista();
	}

	public void actualizaLista() {
		ListView listView = (ListView) findViewById(R.id.listViewActividades);
		AdaptadorCursorActividades adaptador = (AdaptadorCursorActividades) listView.getAdapter();
		adaptador.changeCursor(new TareaBD(this).getActividadesAbiertas(proyecto));
	}

	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
	}

	@Override
	public boolean onItemLongClick(AdapterView<?> parent, View view, int position, final long id) {
		AlertDialog opciones = new AlertDialog.Builder(this)
			.setAdapter(new AdaptadorOperacionesContextuales(this,
			                                                 android.R.layout.select_dialog_item,
			                                                 android.R.id.text1),
			            new DialogInterface.OnClickListener() {
				            public void onClick(DialogInterface dialog,
				                                int selected) {
					            if (selected == 0) {
						            Intent i = new Intent(getBaseContext(), EditorActividades.class);
						            i.putExtra("id_actividad", id);
						            i.putExtra("id_proyecto", proyecto.getId());
						            i.putExtra("proyecto", proyecto.getNombre());
						            startActivityForResult(i, EDITOR_ACTIVIDADES);
					            } else if (selected == 1) {
						            TareaBD bd = new TareaBD(getBaseContext());
						            bd.cerrarActividad(id);
						            actualizaLista();
					            } else if (selected == 2) {
						            TareaBD bd = new TareaBD(getBaseContext());
						            bd.borrarActividad(id);
						            bd.borrarTareasDeActividad(proyecto.getId(), id);
						            actualizaLista();
					            }
				            }
			            }).create();
		opciones.show();
		return true;
	}

	protected class AdaptadorCursorActividades extends CursorAdapter {
		TextView titulo, subtitulo;

		public AdaptadorCursorActividades(Context contexto, Cursor c) {
			super(contexto, c, false);
		}

		@Override
		public View newView(Context contexto, Cursor c, ViewGroup padre) {
			LayoutInflater inflador = (LayoutInflater) contexto.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			return inflador.inflate(R.layout.elemento_actividad, padre, false);
		}

		@Override
		public void bindView(View vista, Context contexto, Cursor c) {
			Button button = (Button) vista.findViewById(R.id.actividad_iniciar_tarea);
			button.setOnClickListener(new IniciarTareaButtonHandler(contexto, c));
			titulo = (TextView) vista.findViewById(R.id.titulo_actividad);
			subtitulo = (TextView) vista.findViewById(R.id.subtitulo_actividad);
			titulo.setText(c.getString(c.getColumnIndex("nombre")));
			subtitulo.setText(c.getString(c.getColumnIndex("descripcion")));
		}
	}

	protected class IniciarTareaButtonHandler implements View.OnClickListener {

		protected long idProyecto;
		protected long idActividad;
		protected Context context;

		public IniciarTareaButtonHandler(Context context, Cursor c) {
			idProyecto = c.getLong(c.getColumnIndex("id_proyecto"));
			idActividad = c.getLong(c.getColumnIndex("_id"));
			this.context = context.getApplicationContext();
		}

		@Override
		public void onClick(View v) {
			stopService(new Intent(context, ServicioCronometro.class));
			Intent i = new Intent(context, ServicioCronometro.class);
			i.putExtra("id_proyecto", idProyecto);
			i.putExtra("id_actividad", idActividad);
			startService(i);
		}
	}

}
