package com.quiomputing.cronopartes.services;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.IBinder;
import android.support.annotation.Nullable;
import android.support.v4.app.NotificationCompat;
import android.util.Log;

import com.quiomputing.cronopartes.R;
import com.quiomputing.cronopartes.actividades.CronoTask;
import com.quiomputing.cronopartes.actividades.DialogoEdicionDescripcion;
import com.quiomputing.cronopartes.bd.TareaBD;
import com.quiomputing.cronopartes.constantes.Constantes;
import com.quiomputing.cronopartes.data.Tarea;

import java.util.Date;

/**
 * Created by joaqu on 09/11/2015.
 */
public class ServicioCronometro extends Service implements Constantes{

	protected long idActividad;
	protected long idProyecto;
	protected Tarea tarea;
	protected TareaBD tareaBD;


	@Override
	public void onCreate() {
		super.onCreate();
	}

	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		if (intent == null)
			return START_NOT_STICKY;
		Bundle extras = intent.getExtras();
		if (extras == null)
			return START_NOT_STICKY;
		NotificationManager notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
		notificationManager.cancel(ID_NOTIFICACION);
		idProyecto = extras.getLong("id_proyecto");
		idActividad = extras.getLong("id_actividad");
		tareaBD = new TareaBD(this);
		tarea = tareaBD.crearTareaNueva(new TareaBD(this).getActividad(idActividad));
		ejecutarCronometroTarea(tarea.getId());
		notificationManager.notify(ID_NOTIFICACION, generateNotification(new TareaBD(this)));
		return START_STICKY;
	}

	protected Notification generateNotification(TareaBD bd) {
		Intent editorIntent = new Intent(this, DialogoEdicionDescripcion.class);
		editorIntent.putExtra("id", tarea.getId());
		NotificationCompat.Builder notifc = new NotificationCompat.Builder(this)
//			.setStyle(new NotificationCompat.BigTextStyle().bigText("Este es un texto de ejemplo"))
			.setContentTitle(getResources().getString(R.string.chronometer))
			.setSmallIcon(R.mipmap.ic_launcher)
			.setContentText(bd.getActividad(idActividad).getNombre())
			.setContentIntent(PendingIntent.getActivity(this, 0, new Intent(this, CronoTask.class), 0))
			.setLargeIcon(BitmapFactory.decodeResource(getResources(), R.mipmap.ic_launcher))
			.setWhen(System.currentTimeMillis() + 1000 * 60 * 60)
			.addAction(android.R.drawable.ic_menu_edit,
			           getResources().getString(R.string.actividad_label_descripcion),
			           PendingIntent.getActivity(this, 0, editorIntent, PendingIntent.FLAG_UPDATE_CURRENT))
			.addAction(android.R.drawable.ic_menu_close_clear_cancel,
			           getResources().getString(R.string.end),
			           PendingIntent.getBroadcast(this, 0, new Intent("com.quiomputing.cronopartes.cancel"), PendingIntent.FLAG_CANCEL_CURRENT));
		Notification notification = notifc.build();
		notification.flags |= Notification.FLAG_NO_CLEAR;
		notification.flags |= Notification.FLAG_ONGOING_EVENT;
		return notification;
	}

	@Override
	public void onDestroy() {
		actualizarTarea(tarea);
		NotificationManager notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
		notificationManager.cancel(ID_NOTIFICACION);
	}

	@Nullable
	@Override
	public IBinder onBind(Intent intent) { return null; }

	public void ejecutarCronometroTarea(long idTarea) {
		CronometroTarea cronometro = new CronometroTarea();
		cronometro.execute(idTarea);
	}

	public void actualizarTarea(Tarea tarea) {
		if (tarea == null)
			return;
		tarea.setFechaFin(new Date(System.currentTimeMillis()));
		Date fechaInicio = tarea.getFechaInicio();
		Date fechaFin = tarea.getFechaFin();
		long diferencia = fechaFin.getTime() - fechaInicio.getTime();
		// tiempo en segundos
		tarea.setTiempo(diferencia / 1000);
		tareaBD.guardar(tarea);
	}

	public class CronometroTarea extends AsyncTask<Long, Void, Void> {

		@Override
		protected Void doInBackground(Long... idTarea) {
			TareaBD tareaBD = new TareaBD(ServicioCronometro.this);
			do {
				Tarea temp = tareaBD.getTarea(idTarea[0]);
				actualizarTarea(temp);
				try {
					Thread.sleep(TIEMPO_ENTRE_ACTUALIZACIONES);
				} catch (InterruptedException e) {
					Log.e(ServicioCronometro.this.getClass().getSimpleName(), "Error en thread", e);
					return null;
				}
			} while (true);
		}
	}


}
